/************************************************************************/
/* dlist.h								*/
/************************************************************************/
/*
 *  (c)2023 Todd Shadburn <tshadburn@opencomputingsolutions.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#ifndef _DLIST_H_
#define _DLIST_H_

#include <stdint.h>
#include <assert.h>


typedef struct _dlist_t dlist_t;
struct _dlist_t {
	dlist_t		*prev;
	dlist_t		*next;
};



static inline void dlist_init(dlist_t *item)
{
	item->prev=item;
	item->next=item;
	return;
}


static inline void __dlist_add(dlist_t *item,
				dlist_t *prev,
				dlist_t *next)
{
	next->prev = item;
	item->next = next;
	item->prev = prev;
	prev->next = item;
}


// prepend the new item to the list (stack-like behavior)
static inline void dlist_add(dlist_t *list, dlist_t *item)
{
	assert(list->next != item);
	__dlist_add(item, list, list->next);
}


// append the new item to the list (queue-like behavior)
static inline void dlist_add_tail(dlist_t *list, dlist_t *item)
{
	__dlist_add(item, list->prev, list);
}


static inline void __dlist_remove(dlist_t *prev, dlist_t *next)
{
	next->prev = prev;
	prev->next = next;
}


static inline void dlist_remove(dlist_t *item)
{
	__dlist_remove(item->prev, item->next);
}


static inline int dlist_is_empty(dlist_t *list)
{
	return list->next == list;
}


#define dlist_for_each(pos, list) \
	for (pos=(list)->next ; pos!=list ; pos=pos->next)

#define dlist_for_each_prev(pos, list) \
	for (pos=(list)->prev ; pos!=list ; pos=pos->prev)



#endif

