/************************************************************************/
/* udp.c								*/
/************************************************************************/
/*
 *  (c)2021 Todd Shadburn <tshadburn@opencomputingsolutions.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#include <stdint.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include <netinet/in.h>
#include <netdb.h>
#include <time.h>
#include <fcntl.h>
#include <signal.h>

#include <udp.h>
#include <helpers.h>



int udp_bind(char *address, char *service)
{
	int			rc;
	int			s;
	int			flags;
	struct addrinfo		*addr_list;

	rc = getaddrinfo(address, service, NULL, &addr_list);
	if (rc != 0) {
//		log_entry("Failed to lookup hostname errno=%d\n", errno);
		return -1;
	}
	s = socket(AF_INET, SOCK_DGRAM, 0);
	if (s == -1) {
//		log_entry("Failed to create UDP socket errno=%d\n", errno);
		freeaddrinfo(addr_list);
		return -1;
		}
	flags = fcntl(s, F_GETFL, 0);
	rc = fcntl(s, F_SETFL, flags | O_NONBLOCK);
	if (rc == -1) {
//		log_entry("Failed to set O_NONBLOCK on socket errno=%d\n", errno);
		freeaddrinfo(addr_list);
		return -1;
	}
	rc = bind(s, addr_list->ai_addr, addr_list->ai_addrlen);
	if (rc != 0) {
//		log_entry("Failed to bind to UDP socket errno=%d\n", errno);
		freeaddrinfo(addr_list);
		return -1;
	}
	freeaddrinfo(addr_list);

	return s;
}


int udp_connect(char *address, char *service)
{
	int			rc;
	int			s;
	int			flags;
	struct addrinfo		*addr_list;

	rc = getaddrinfo(address, service, NULL, &addr_list);
	if (rc != 0) {
//		log_entry("Failed to lookup hostname errno=%d\n", errno);
		return -1;
	}
	s = socket(AF_INET, SOCK_DGRAM, 0);
	if (s == -1) {
//		log_entry("Failed to create UDP socket errno=%d\n", errno);
		freeaddrinfo(addr_list);
		return -1;
		}
	flags = fcntl(s, F_GETFL, 0);
	rc = fcntl(s, F_SETFL, flags | O_NONBLOCK);
	if (rc == -1) {
//		log_entry("Failed to set O_NONBLOCK on socket errno=%d\n", errno);
		freeaddrinfo(addr_list);
		return -1;
	}
	rc = connect(s, addr_list->ai_addr, addr_list->ai_addrlen);
	if (rc != 0) {
//		log_entry("Failed to connect UDP socket errno=%d\n", errno);
		freeaddrinfo(addr_list);
		return -1;
	}
	freeaddrinfo(addr_list);

	return s;
}


int udp_shutdown(int socket)
{
	int	rc;
	rc = shutdown(socket, SHUT_RDWR);
	rc = close(socket);
	return rc;
}
 

