/************************************************************************/
/* packetbuf.h - Packetbuf module					*/
/************************************************************************/
/*
 *  (c)2021,2022,2024 Todd Shadburn <tshadburn@opencomputingsolutions.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#ifndef _PACKETBUF_H_
#define _PACKETBUF_H_


#include <stdint.h>
#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <dlist.h>


#define PACKET_BUFFER_SIZE	16384


typedef struct _packetbuf_t packetbuf_t;
struct _packetbuf_t {
	dlist_t		list;

	pnetbuffer_t	*nb;
	char		nb_buffer[PACKET_BUFFER_SIZE];

//	int		sfd;

	struct sockaddr	saddr;
	socklen_t	saddr_len;

	void		*data;
};


typedef struct _packetbuf_slab_t packetbuf_slab_t;
struct _packetbuf_slab_t {
	uint32_t	size_max;
	uint32_t	size_curr;
	dlist_t		head;
};


packetbuf_slab_t *packetbuf_slab_alloc(uint32_t initsize, uint32_t maxsize);
int packetbuf_slab_free(packetbuf_slab_t *slab);
packetbuf_t *packetbuf_slab_get(packetbuf_slab_t *slab);
void packetbuf_slab_return(packetbuf_slab_t *slab, packetbuf_t *pb);
void packetbuf_set_address(packetbuf_t *pb,
			struct sockaddr *sa,
			socklen_t sa_len);
void packetbuf_get_address(packetbuf_t *pb,
			struct sockaddr *sa,
			socklen_t *sa_len);



#ifdef PACKETBUF_STATIC_FUNCS
static inline void packetbuf_set_data(packetbuf_t *pb, void *data)
{
	assert(pb);
	pb->data = data;
	return;
}

static inline void *packetbuf_get_data(packetbuf_t *pb)
{
	assert(pb);
	return pb->data;
}


#else
void packetbuf_set_data(packetbuf_t *pb, void *data);
void *packetbuf_get_data(packetbuf_t *pb);

#endif


// inline debug functions
static inline void dump_packetbuf(packetbuf_t *pb)
{
	printf("%s() pb=%p", __func__, pb);
	if (!pb) return;
//	printf("%s() .list=%p .sfd=%d .req=%p\n",
	printf("%s() .list=%p .req=%p\n",
		__func__,
		&pb->list,
//		pb->sfd,
		pb->data);
	return;
}


#endif


